# Common OIDs Query Module 
# A perl module for querying some common SNMP OIDs
#
# written by Scott Matott sXe (matott@mcs.anl.gov)


package coqm;

$VERSION = '0.0.1';  # alpha code for your monkey

require Exporter;
require DynaLoader;
require AutoLoader;

use strict;
use Socket;
use SNMP_util;

@coqm::ISA = qw(Exporter AutoLoader DynaLoader);

# functions to export
@coqm::EXPORT = qw(create_session, set_session, get, walk, fetch_ifDescr, fetch_ifName);


# suppress snmp warnings
$SNMP_Session::suppress_warnings = 2;


# OIDs 

# Device stuff
my $sysDescr = "1.3.6.1.2.1.1.1.0";
my $sysName = "1.3.6.1.2.1.1.5.0";


# Basic interface stuff (mostly from RFC standard IF-MIB)
my $ifDescr = "1.3.6.1.2.1.2.2.1.2"; # interface description/name from the RFC MIBs
my $ifName = "1.3.6.1.2.1.31.1.1.1.1"; # interface name from Cisco MIBs (seems to work on other vendors as well)
my $ifType = "1.3.6.1.2.1.2.2.1.3";
my $ifMtu = "1.3.6.1.2.1.2.2.1.4";
my $ifSpeed = "1.3.6.1.2.1.2.2.1.5";
my $ifPhysAddress = "1.3.6.1.2.1.2.2.1.6";
my $ifAdminStatus = "1.3.6.1.2.1.2.2.1.7";
my $ifOperStatus = "1.3.6.1.2.1.2.2.1.8";
my $ifLastChange = "1.3.6.1.2.1.2.2.1.9";

# some arrays related to the above
my @ifAdminStati = ( "NULL", "up", "down", "testing");
my @ifOperStati = ( "NULL", "up", "down", "testing", "unknown", "dormant", "notPresent", "lowerLayerDown", );

# counters
my $ifInOctets = "1.3.6.1.2.1.2.2.1.10";
my $ifInUcastPkts = "1.3.6.1.2.1.2.2.1.11";
my $ifInNUcastPkts = "1.3.6.1.2.1.2.2.1.12";
my $ifInDiscards = "1.3.6.1.2.1.2.2.1.13";
my $ifInErrors = "1.3.6.1.2.1.2.2.1.14";
my $ifInUnknownProtos = "1.3.6.1.2.1.2.2.1.15";
my $ifOutOctets = "1.3.6.1.2.1.2.2.1.16";
my $ifOutUcastPkts = "1.3.6.1.2.1.2.2.1.17";
my $ifOutNUcastPkts = "1.3.6.1.2.1.2.2.1.18";
my $ifOutDiscards = "1.3.6.1.2.1.2.2.1.19";
my $ifOutErrors = "1.3.6.1.2.1.2.2.1.20";
my $ifOutQLen = "1.3.6.1.2.1.2.2.1.21";
my $ifSpecific = "1.3.6.1.2.1.2.2.1.22";

# High Capacity Counter (these require use of BigInt, since they are 64 bit numbers)
my $ifHCInOctets = "1.3.6.1.2.1.31.1.1.1.6";
my $ifHCInUcastPkts = "1.3.6.1.2.1.31.1.1.1.7";
my $ifHCInMulticastPkts = "1.3.6.1.2.1.31.1.1.1.8";
my $ifHCInBroadcastPkts = "1.3.6.1.2.1.31.1.1.1.9";
my $ifHCOutOctets = "1.3.6.1.2.1.31.1.1.1.10";
my $ifHCOutUcastPkts = "1.3.6.1.2.1.31.1.1.1.11";
my $ifHCOutMulticastPkts = "1.3.6.1.2.1.31.1.1.1.12";
my $ifHCOutBroadcastPkts = "1.3.6.1.2.1.31.1.1.1.13";

# When were the counters last cleared? gives hundredths of a second since coutners were cleared
my $sysClearPortTime = "1.3.6.1.4.1.9.5.1.1.13";

# Vlan Stuff
my $vmMembershipSummaryTable = "1.3.6.1.4.1.9.9.68.1.2.1"; # provides a list of all vlans on device
my $vtpVlanName = "1.3.6.1.4.1.9.9.46.1.3.1.1.4"; # provides text descrpition of the vlan(s)
my $vmVlan = "1.3.6.1.4.1.9.9.68.1.2.2.1.2"; # provides access vlan for ifindex(es)


# Trunk Port Stuff
my $vlanTrunkPortEncapsulationType  = "1.3.6.1.4.1.9.9.46.1.6.1.1.3";
my $vlanTrunkPortVlansEnabled  = "1.3.6.1.4.1.9.9.46.1.6.1.1.4"; # .ifindex
my $vlanTrunkPortNativeVlan    = "1.3.6.1.4.1.9.9.46.1.6.1.1.5"; # .ifindex
my $vlanTrunkPortDynamicState  = "1.3.6.1.4.1.9.9.46.1.6.1.1.13";
my $vlanTrunkPortDynamicStatus = "1.3.6.1.4.1.9.9.46.1.6.1.1.14"; # 1=trunk, 2=nottrunk  NULL=access or what
my $vlanTrunkPortEncapsulationOperType = "1.3.6.1.4.1.9.9.46.1.6.1.1.16";

# Stp Status Stuff
my $dot1dStpPortState = "1.3.6.1.2.1.17.2.15.1.3"; # .bridgeid (query with community@vlan)
my @StpStates = ( "NULL", "disabled", "blocking", "listening", "learning", "forwarding", "broken" );


# arrays for the above
my @vlanTrunkPortDynamicStati = ( "access", "trunk", "access",);
my @vlanTrunkPortDynamicStates = ( "", "on", "off", "desirable", "auto", "onNoNegotiate",);
my @vlanTrunkPortEncapsulationTypes = ( "", "isl", "dot10", "lane", "dot1Q", "negotiate",);
my @vlanTrunkPortEncapsulationOperTypes = ( "", "isl", "dot10", "lane", "dot1Q", "negotiating", "notApplicable");


# Cam Table stuff (also called 'bridgeport' stuff)
# Cisco cam OIDs, these need to be walked 'per vlan', that is with community@vlan for a community string
my $dot1dBasePortIfIndex = "1.3.6.1.2.1.17.1.4.1.2"; # converts bridgeids to ifindices
my $dot1dTpFdbPort = "1.3.6.1.2.1.17.4.3.1.2"; # nasty little oids, the oid contains the mac addr, the value is a bridgeport
my $dot1dTpFdbStatus = "1.3.6.1.2.1.17.4.3.1.3" ; # how was the mac obtained
my $dot1dTpFdbStati = ( "unknown", "other", "invalid", "learned", "self", "mgmt",);

# Extreme cam OIDs
my $extreme_cam_macs_oid = "1.3.6.1.4.1.1916.1.16.1.1.3";
my $extreme_cam_ifindex_oid = "1.3.6.1.4.1.1916.1.16.1.1.4";


# CDP oids
my $cdpGlobalDeviceId  = "1.3.6.1.4.1.9.9.23.1.3.4.0";
my $cdpCacheAddress    = "1.3.6.1.4.1.9.9.23.1.2.1.1.4"; # .index.n
my $cdpCacheDeviceId   = "1.3.6.1.4.1.9.9.23.1.2.1.1.6"; # .index.n
my $cdpCacheDevicePort = "1.3.6.1.4.1.9.9.23.1.2.1.1.7"; # .index.n
my $cdpCachePlatform   = "1.3.6.1.4.1.9.9.23.1.2.1.1.8"; # .index.n
my $cdpCacheNativeVLAN = "1.3.6.1.4.1.9.9.23.1.2.1.1.11";
my $cdpCacheDuplex     = "1.3.6.1.4.1.9.9.23.1.2.1.1.12";


# Routing tables
my $ipNetToMediaPhysAddress = "1.3.6.1.2.1.4.22.1.2"; # official arp tables via RFC1213, format is .ifindex.ip
my $atPhysAddress = "1.3.6.1.2.1.3.1.1.2"; # Also from RFC1213-MIB, seems to  give arp tables as well, format is .ifindex.1.ip
my $ipRouteDest    = "1.3.6.1.2.1.4.21.1.1";
my $ipRouteIfIndex = "1.3.6.1.2.1.4.21.1.2";
my $ipRouteNextHop = "1.3.6.1.2.1.4.21.1.7";
my $ipRouteMask    = "1.3.6.1.2.1.4.21.1.11";
my $ipRouteType    = "1.3.6.1.2.1.4.21.1.8";

my @ipRouteTypes   = ( "other", "other", "invalid", "direct", "indirect",);


# cisco Hot Stand-by Router stuff
my $cHsrpGrpActiveRouter    = "1.3.6.1.4.1.9.9.106.1.2.1.1.11"; # ip of current active router for a network
my $cHsrpGrpStandbyState    = "1.3.6.1.4.1.9.9.106.1.2.1.1.15"; # router's standby status for a network

my $cHsrpGrpStandbyStates   = ( "null", "initial", "learn", "listen", "speak", "standby", "active", );



my $rh_session; # our global session pointer, wherever this points is where the queries go
my $dflt_community = "public"; # alter this to store your community string in a single place

sub create_session
{
	my ( $ip, $community, $port, $timeout, $retries, $backoff, $version ) = (@_);

	# default options = 161:2:3:1:1
	if ( ! $community ) { $community = $dflt_community; }
	if ( ! $port ) { $port = 161; }
	if ( ! $timeout ) { $timeout = 2; }
	if ( ! $retries ) { $retries = 3; }
	if ( ! $backoff ) { $backoff = 1; }
	if ( ! $version ) { $version = 1; }

	my %sess;
	$sess{ 'ip' } = $ip;
	$sess{ 'community' } = $community;
	$sess{ 'port' } = $port;
	$sess{ 'timeout' } = $timeout;
	$sess{ 'retries' } = $retries;
	$sess{ 'backoff' } = $backoff;
	$sess{ 'version' } = $version;

	$sess{ 'session_tail' } = join( ":", $ip, $port, $timeout, $retries, $backoff, $version );
	$sess{ 'session' } = join( "\@", $sess{ 'community' }, $sess{ 'session_tail' } );
	$rh_session = \%sess;
	return \%sess;
}

sub set_session
{
	my ($rh_sess) = (@_);
	my $rh_old_session = $rh_session;
	$rh_session = $rh_sess;
	return $rh_old_session;
}

sub get_session
{
	return $rh_session;
}

# IMHO the walk and get funtions that SNMP_util provides are not so hot,
# guess I like the Net::SNMP style better, so here are some 
# routines to make it better
# these are also useful because they take care of the community@vlan style queries
# for you as well, just pass the vlan you need to query at in
sub get
{
	my ($oid, $vlan) = (@_);
	my $session_string = $$rh_session{ 'session' };
	if ( $vlan )
	{
		$session_string =  $$rh_session{ 'community' } . "\@${vlan}\@" . $$rh_session{ 'session_tail' };
	}
	my($value) = snmpget( $session_string, $oid );
	return $value;
}

sub walk
{
	my ($oid, $vlan, $use_ucd_snmpwalk) = (@_);
	if ( $$rh_session{ 'use_ucd_snmpwalk' } || $use_ucd_snmpwalk ) { return ucd_snmpwalk( $oid, $vlan ); }
	my $session_string = $$rh_session{ 'session' };
	if ( $vlan )
	{
		$session_string =  $$rh_session{ 'community' } . "\@${vlan}\@" . $$rh_session{ 'session_tail' };
	}
	my @lines = snmpwalk( $session_string, $oid );

	my %return;
	foreach my $line (@lines)
	{
		my ($oid, $value ) = split( /:/, $line, 2 );
		$return{ $oid } = $value;
	}
	return \%return;
}

# internal function, designed to handle tables indexed by infindices
sub fetch_from_ifindex_table
{
	my ($oid, $ifindex, $ra_vals, $vlan) = (@_);
	if ( $ifindex ne "" )
	{
		my $value = get( "${oid}.${ifindex}", $vlan );
		if ( ! $ra_vals )
		{
			return $value;
		}
		else
		{
			return $$ra_vals[ $value ];
		}
	}

	# if not given an ifindex, then walk and return entire table
	my $rh_oids = walk( $oid, $vlan );
	my %return;
	# to make more useful, strip out the base oid and
	# return a function indexed by just the ifindices
	foreach my $full_oid (keys %$rh_oids)
	{
		my ( $ifindex, $value ) =  ( $full_oid, $$rh_oids{ $full_oid } );
		$ifindex =~ s/.*\.([0-9]*)$/$1/g;
		if ( ! $ra_vals )
		{
			$return{ $ifindex } = $value;
		}
		else
		{
			$return{ $ifindex } = $$ra_vals[ $value ];
		}
	}
	return \%return;
}


sub fetch_ifDescr
{
	my ($ifindex) = (@_);
	return fetch_from_ifindex_table( $ifDescr, $ifindex );
}

sub fetch_ifName
{
	my ($ifindex) = (@_);
	return fetch_from_ifindex_table( $ifName, $ifindex );
}

sub fetch_ifOperStatus
{
	my ($ifindex) = (@_);
	return fetch_from_ifindex_table( $ifOperStatus, $ifindex, \@ifOperStati);
}

sub fetch_ifAdminStatus
{
	my ($ifindex) = (@_);
	return fetch_from_ifindex_table( $ifOperStatus, $ifindex, \@ifAdminStati);
}

sub fetch_ifSpeed
{
	my ($ifindex) = (@_);
	return fetch_from_ifindex_table( $ifSpeed,  $ifindex );
}


sub fetch_ifPhysAddress
{
	my ($ifindex) = (@_);
	my $rh = fetch_from_ifindex_table( $ifPhysAddress, $ifindex );
	if ( $ifindex ne "" )
	{
		return unpack_mac( $rh );
	}
	foreach my $oid (keys %$rh)
	{
		$$rh{ $oid } = unpack_mac( $$rh{ $oid } );
	}
	return $rh;
}

sub fetch_ifLastChange
{
	my ($ifindex) = (@_);
	fetch_from_ifindex_table( $ifLastChange, $ifindex );
}

# Some vlan stuff (all cisco specifc OIDs for now I'm afraid)
sub fetch_ifAccessVlan
{
	my ($ifindex) = (@_);
	return fetch_from_ifindex_table( $vmVlan, $ifindex );
}

sub fetch_vlanTrunkPortNativeVlan
{
	my ($ifindex) = (@_);
	return fetch_from_ifindex_table( $vlanTrunkPortNativeVlan, $ifindex );
}


# internal helper function for dealing with these
# funky 'bit pattern' return values that some of the
# vlan related cisco oids return
sub ParseVlanBitPattern
{
	my ($VlanBitPattern) = (@_);
	$VlanBitPattern = unpack('B*', $VlanBitPattern );
	my $vlan = 0;
	my @vlans;
	foreach my $bit ( split( '', $VlanBitPattern ) )
	{
		if ( $bit eq "1" )
		{
			push(@vlans, $vlan);
		}
		$vlan++;
	}
	return \@vlans;
}

sub fetch_vlanTrunkPortVlansEnabled
{
	my ($ifindex) = (@_);
	my $rh = fetch_from_ifindex_table( $vlanTrunkPortVlansEnabled, $ifindex );
	if ($ifindex ne "")
	{
		return ParseVlanBitPattern( $rh );
	}
	foreach my $ifindex (keys %{$rh})
	{
		$$rh{ $ifindex } = ParseVlanBitPattern( $$rh{ $ifindex } );
	}
	return $rh;
}

sub fetch_vlanTrunkPortDynamicStatus
{
	my ($ifindex) = (@_);
	return fetch_from_ifindex_table( $vlanTrunkPortDynamicStatus, $ifindex, \@vlanTrunkPortDynamicStati );
}

sub fetch_vlanTrunkPortDynamicState
{
	my ($ifindex) = (@_);
	return fetch_from_ifindex_table( $vlanTrunkPortDynamicState, $ifindex, \@vlanTrunkPortDynamicStates );
}

sub fetch_vlanTrunkPortEncapsulationType
{
	my ($ifindex) = (@_);
	return fetch_from_ifindex_table( $vlanTrunkPortEncapsulationType, $ifindex, \@vlanTrunkPortEncapsulationTypes );
}

sub fetch_vlanTrunkPortEncapsulationOperType
{
	my ($ifindex) = (@_);
	return fetch_from_ifindex_table( $vlanTrunkPortEncapsulationOperType, $ifindex, \@vlanTrunkPortEncapsulationOperTypes );
}



# low (32 bit) counters
sub fetch_ifInOctets
{
	my ($ifindex) = (@_);
	return fetch_from_ifindex_table( $ifInOctets, $ifindex );
}

sub fetch_ifInUcastPkts
{
	my ($ifindex) = (@_);
	return fetch_from_ifindex_table( $ifInUcastPkts, $ifindex );
}

sub fetch_ifInNUcastPkts
{
	my ($ifindex) = (@_);
	return fetch_from_ifindex_table( $ifInNUcastPkts, $ifindex );
}

sub fetch_ifInDiscards
{
	my ($ifindex) = (@_);
	return fetch_from_ifindex_table( $ifInDiscards, $ifindex );
}

sub fetch_ifInErrors
{
	my ($ifindex) = (@_);
	return fetch_from_ifindex_table( $ifInErrors, $ifindex );
}

sub fetch_ifInUnknownProtos
{
	my ($ifindex) = (@_);
	return fetch_from_ifindex_table( $ifInUnknownProtos, $ifindex );
}

sub fetch_ifOutOctets
{
	my ($ifindex) = (@_);
	return fetch_from_ifindex_table( $ifOutOctets, $ifindex );
}

sub fetch_ifOutUcastPkts
{
	my ($ifindex) = (@_);
	return fetch_from_ifindex_table( $ifOutUcastPkts, $ifindex );
}

sub fetch_ifOutNUcastPkts
{
	my ($ifindex) = (@_);
	return fetch_from_ifindex_table( $ifOutNUcastPkts, $ifindex );
}

sub fetch_ifOutDiscards
{
	my ($ifindex) = (@_);
	return fetch_from_ifindex_table( $ifOutDiscards, $ifindex );
}

sub fetch_ifOutErrors
{
	my ($ifindex) = (@_);
	return fetch_from_ifindex_table( $ifOutErrors, $ifindex );
}

sub fetch_ifOutQLen
{
	my ($ifindex) = (@_);
	return fetch_from_ifindex_table( $ifOutQLen, $ifindex );
}

sub fetch_ifSpecific
{
	my ($ifindex) = (@_);
	return fetch_from_ifindex_table( $ifSpecific, $ifindex );
}

sub fetch_sysClearPortTime
{
	my ($ifindex) = (@_);
	return fetch_from_ifindex_table( $sysClearPortTime, $ifindex );
}



# another internal function, this one make sure to use snmp version 2c or higher
# which is required for retrieving 64 bit counter values
sub fetch_HCoid
{
	my ($oid, $ifindex) = (@_);
	my $old_version = $$rh_session{ 'version' }; 
	if ( $old_version == 1 )
	{
		create_session( $$rh_session{ 'ip' }, $$rh_session{ 'community' }, $$rh_session{ 'port' }, $$rh_session{ 'timeout' }, $$rh_session{ 'retries' }, $$rh_session{ 'backoff' }, "2c" );
		
	}
	my $return = fetch_from_ifindex_table( $oid, $ifindex);
	# restore the session if need be
	if ( $old_version == 1 )
	{
		create_session( $$rh_session{ 'ip' }, $$rh_session{ 'community' }, $$rh_session{ 'port' }, $$rh_session{ 'timeout' }, $$rh_session{ 'retries' }, $$rh_session{ 'backoff' }, $old_version );
	}
	return $return;
}

# High (64 bit) counters
sub fetch_ifHCInOctets
{
	my ($ifindex) = (@_);
	return fetch_HCoid( $ifHCInOctets, $ifindex );
}

sub fetch_ifHCInUcastPkts
{
	my ($ifindex) = (@_);
	return fetch_HCoid( $ifHCInUcastPkts, $ifindex );
}

sub fetch_ifHCInMulticastPkts
{
	my ($ifindex) = (@_);
	return fetch_HCoid( $ifHCInMulticastPkts, $ifindex );
}

sub fetch_ifHCInBroadcastPkts
{
	my ($ifindex) = (@_);
	return fetch_HCoid( $ifHCInBroadcastPkts, $ifindex );
}

sub fetch_ifHCOutOctets
{
	my ($ifindex) = (@_);
	return fetch_HCoid( $ifHCOutOctets, $ifindex );
}

sub fetch_ifHCOutUcastPkts
{
	my ($ifindex) = (@_);
	return fetch_HCoid( $ifHCOutUcastPkts, $ifindex );
}

sub fetch_ifHCOutMulticastPkts
{
	my ($ifindex) = (@_);
	return fetch_HCoid( $ifHCOutMulticastPkts, $ifindex );
}

sub fetch_ifHCOutBroadcastPkts
{
	my ($ifindex) = (@_);
	return fetch_HCoid( $ifHCOutBroadcastPkts, $ifindex );
}

sub fetch_cdpGlobalDeviceId
{
	return get( $cdpGlobalDeviceId );
}

# another internal function, this one handles queries from the cisco CDP oid tree
# these tables are indexed like so: oid.ifindex.process_id = value
sub fetch_from_cdp_tree
{
	my ($oid, $ifindex, $pid) = (@_);
	if ( $ifindex ne "" )
	{
		$oid = $oid . "." . $ifindex;
		if ( $pid ne "" )
		{
			$oid = $oid . "." . $pid ;
			return get( $oid );
		}
		return walk( $oid );
	}

	my %return;
	my $rh = walk( $oid );
	foreach my $key (keys %{$rh})
	{
		my ( $ifindex, $pid ) = split( /\./, $key );
		$return{ $ifindex }{ $pid } = $$rh{ $key };
	}
	return \%return;
}

# helper function
# converts a binary ip address to ascii form
# doesn't fail if passed and incomplete binary value
sub safe_inet_ntoa
{
	my ($addr) = (@_);
	if ( length($addr) == 4 )
	{
		return inet_ntoa($addr);
	}
	else
	{
		return "NULL";
	}
}

sub fetch_cdpCacheAddress
{
	my ($ifindex, $pid) = (@_);
	my $rh = fetch_from_cdp_tree( $cdpCacheAddress, $ifindex, $pid );
	if ( $pid ne "" )
	{
		return safe_inet_ntoa( $rh );
	}
	if ( $ifindex ne "" )
	{
		foreach my $pid ( keys %$rh )
		{
			$$rh{ $pid } = safe_inet_ntoa( $$rh{ $pid } );
		}
		return $rh;
	}
	foreach my $ifindex ( keys %$rh )
	{
		foreach my $pid ( keys %{$$rh{ $ifindex }} )
		{
			$$rh{ $ifindex }{ $pid } = safe_inet_ntoa( $$rh{ $ifindex }{ $pid } );
		}
	}
	return $rh;
}


sub fetch_cdpCacheDeviceId
{
	my ($ifindex, $pid) = (@_);
	return fetch_from_cdp_tree( $cdpCacheDeviceId, $ifindex, $pid );
}

sub fetch_cdpCacheDevicePort
{
	my ($ifindex, $pid) = (@_);
	return fetch_from_cdp_tree( $cdpCacheDevicePort, $ifindex, $pid );
}

sub fetch_cdpCachePlatform
{
	my ($ifindex, $pid) = (@_);
	return fetch_from_cdp_tree( $cdpCachePlatform, $ifindex, $pid );
}

sub fetch_cdpCacheNativeVLAN
{
	my ($ifindex, $pid) = (@_);
	return fetch_from_cdp_tree( $cdpCacheNativeVLAN, $ifindex, $pid );
}

sub fetch_cdpCacheDuplex
{
	my ($ifindex, $pid) = (@_);
	return fetch_from_cdp_tree( $cdpCacheDuplex, $ifindex, $pid );
}


# returns a sorted array of vlans configured on the device, also stashed that list for future use
sub fetch_vlans
{
	#my $rh_oids = walk( $vmMembershipSummaryTable );
	my $rh_oids = walk( $vtpVlanName );
	my @unsorted;
	foreach my $oid ( keys %$rh_oids )
	{
		my $vlan = $oid;
		$vlan =~ s/.*\.([0-9]*)$/$1/g;
		if ($vlan > 1000) { next; };
		push(@unsorted, $vlan );
	}
	my @sorted = sort { $a <=> $b } (@unsorted);
	# stash a copy of the array in our sessions hash for future use
	$$rh_session{ 'vlans' } = \@sorted;
	return \@sorted;
}

# internal function, either takes passed vlan and puts in array, or takes list of vlans from session, creating said list if required
sub setup_vlans_array
{
	my ($vlan) = (@_);
	my @vlans;
 	# if not passed a vlan, try getting list of vlans from session
	# if not there, get it by querying the switch
	# if still nothing, then stick an empty value in so we still query
 	if ( ! $vlan )
	{
		my $ra = $$rh_session{ 'vlans' } ;
		if ( ! $ra )
		{
			$ra = fetch_vlans( );
		}
		(@vlans) = (@$ra);
		if ( ! $vlans[0] ) { $vlans[0] = ""; }
	}
	else
	{
		push(@vlans, $vlan);
	}
	return \@vlans;
}

sub fetch_bridgeport_to_ifindex
{
	my ($bridgeport, $vlan) = (@_);

	my $ra_vlans = setup_vlans_array( $vlan );
	my (@vlans) = (@$ra_vlans);
	if ( $bridgeport )
	{
		foreach my $vlan (@vlans)
		{
			my $ifindex = get($dot1dBasePortIfIndex . "." . $bridgeport, $vlan);
			if ( $ifindex ) { return $ifindex; }
		}
		return;
	}
	else
	{
		my %return;
		foreach my $vlan (@vlans)
		{
			my $rh_oids = walk($dot1dBasePortIfIndex, $vlan );
			foreach my $oid ( keys %$rh_oids )
			{
				my $bridgeport = $oid;
				$bridgeport =~ s/.*\.([0-9]*)$/$1/g;
				$return{ $bridgeport } = $$rh_oids{ $oid };
			}
		}
		return \%return;
	}
}

sub fetch_dot1dStpPortState
{
	my ( $bridgeport, $ra_vlans ) = (@_);
	my $ra_all_vlans = setup_vlans_array();
	my %all_vlans;
	foreach my $key (@$ra_all_vlans)
	{
		$all_vlans{ $key } = "YES";
	}

	if ( $ra_vlans )
	{
		if ( ! ref( $ra_vlans ) )
		{
			# we were passed a single vlan to check the status of
			if ( exists( $all_vlans{ $ra_vlans } ) )
			{
				return fetch_from_ifindex_table( $dot1dStpPortState, $bridgeport, \@StpStates, $ra_vlans );
			}
			return "NULL";
		}
	}
	else
	{
		$ra_vlans = $ra_all_vlans;
	}

	my %return;
	foreach my $vlan (@$ra_vlans)
	{
		# save the bother of querying for vlans that aren't defined on the device
		# a must since these queries would timeout if we tried them
		if ( exists( $all_vlans{ $vlan } ) )
		{
			$return{ $vlan } = fetch_from_ifindex_table( $dot1dStpPortState, $bridgeport, \@StpStates, $vlan );
		}
		else
		{
			$return{ $vlan } = "NULL";
		}
	}
	return \%return;

}

sub fetch_mac_bridgeport_and_vlan
{
	my ($mac, $vlan) = (@_);
	my $ra_vlans = setup_vlans_array( $vlan );
	my (@vlans) = (@$ra_vlans);
	if ( $mac )
	{
		my $oid = mac_addr_to_oid( $mac );
		my %return;
		foreach my $vlan (@vlans)
		{
			my $bridgeport = get( $dot1dTpFdbPort . $oid, $vlan);
			if ( $bridgeport )
			{
				$return{ 'vlan' } = $vlan;
				$return{ 'bridgeport' } = $bridgeport;
				return \%return;
			}
		}
		return \%return;
	}

	my %return;
	foreach my $vlan (@vlans)
	{
		#my $rh_oids = walk($dot1dTpFdbPort, $vlan);
		my $rh_oids = walk($dot1dTpFdbPort, $vlan);
		foreach my $oid ( keys %$rh_oids )
		{
			my $mac = oid_to_mac_addr( $oid );
			if ( $return{ 'bridgeport' }{ $mac } ) { next; } # skip macs we've seen already (it is possible to see the same mac on two vlans, actually should probly account for that somehow...)
			$return{ 'bridgeport' }{ $mac } =  $$rh_oids{ $oid };
			$return{ 'vlan' }{ $mac } = $vlan ;
			push (@{ $return{ 'by_bridgeport' }{ $$rh_oids{ $oid } } }, $mac);
		}
	}
	return \%return;
}

sub fetch_cisco_cam_table
{
	my $rh_bport2ifindex = fetch_bridgeport_to_ifindex( );
	my $rh_bport2mac = fetch_mac_bridgeport_and_vlan();
	$rh_bport2mac = $$rh_bport2mac{ 'by_bridgeport' };
	my %return;
	foreach my $bridgeport ( keys %$rh_bport2mac )
	{
		my $ifindex = $$rh_bport2ifindex{ $bridgeport };
		my $ra_macs = $$rh_bport2mac{ $bridgeport };
		$return{ 'by_ifindex' }{ $ifindex } = $ra_macs;
		foreach my $mac (@$ra_macs)
		{
			$return{ 'by_mac' }{ $mac } = $ifindex;
		}
	}
	return \%return;
}

sub fetch_extreme_cam_table
{
	my $rh_macs = walk( $extreme_cam_macs_oid );
	my %return;
	foreach my $oid ( keys %$rh_macs )
	{
		my $mac = unpack_mac( $$rh_macs{ $oid } );
		my $last_two = $oid;
		$last_two =~ s/^.*\.([0-9]+)\.([0-9]+)$/$1.$2/g;
		my $ifindex = get( $extreme_cam_ifindex_oid . "." .  $last_two );
		push ( @{ $return{ 'by_ifindex' }{ $ifindex } }, $mac );
		$return{ 'by_mac' }{ $mac } = $ifindex;
	}
	return \%return;
}

sub fetch_cam_table
{
	my ($type) = (@_);
	if ( ! $type )
	{
		my $rh =  fetch_cam_table( "extreme" );
		if ( scalar((keys %$rh)) > 0 ) { return $rh; }
		$rh =  fetch_cam_table( "cisco" );
		if ( scalar((keys %$rh)) > 0 ) { return $rh; }
	}
	if ( $type eq "cisco" )
	{
		return fetch_cisco_cam_table();
	}
	if ( $type eq "extreme" )
	{
		return fetch_extreme_cam_table();
	}
	my %empty;
	return \%empty;
}


sub fetch_arp_table
{
	my ($ifindex) = (@_);
	my $oid = $ipNetToMediaPhysAddress;
	if ( $ifindex )
	{
		#$oid = $oid . "." . $ipNetToMediaPhysAddress;
		$oid = $oid . "." . $ifindex;
	}
	
	my %return;
	my $rh_table = walk( $oid );
	foreach my $line (keys %$rh_table)
	{
		my $ip = $line;
		$ip =~ s/^$oid\.//g;
		if ( ! $ifindex )
		{
			$ip =~ s/^[0-9]+\.//g;
		}
		my $mac = unpack_mac( $$rh_table{ $line } );
		push (@{ $return{ 'by_mac' }{ $mac } },$ip );
		$return{ 'by_ip' }{ $ip } = $mac;
	}
	return \%return;

}

# Misc helper functions

# takes a mac and makes an oid out of it
sub mac_addr_to_oid
{
	my ($mac) = (@_);
	$mac = lc($mac);
	$mac =~ s/[^a-f0-9]//g;
	$mac =~ s/(..)/$1./g;
	$mac =~ s/\.$//g;
	my @pairs = split( /\./, $mac );
	my $oid;
	foreach my $hex (@pairs)
	{
		my $num = unpack( "C*", pack("H*", $hex));
		$oid = $oid . "." . $num ;
	}
	return $oid;
}

sub oid_to_mac_addr
{
	my ($macoid) = (@_);
	$macoid =~ s/^1\.3\.6\.1\.2\.1\.17\.4\.3\.1\.2//g;
	my @words = split( /\./ , $macoid );
	my $mac;
	my $spot = 0;
	foreach my $word (@words)
	{
		my $hex = sprintf ("%2x",$word );
		# this is just some stuff to make sure we don't put a : on the first hex number
		if ( $spot )
		{
			$mac = $mac . ":" . $hex;
		}
		else
		{
			$mac = $hex;
		}
		$spot++;
	}
	# convert form
	$mac =~ s/ /0/g;
	$mac =~ s/^(..):(..):(..):(..):(..):(..)$/$1$2\.$3$4\.$5$6/g;
	return $mac;
}

sub unpack_mac
{
	my ($mac) = (@_);
	$mac = unpack( "H*", $mac );
	$mac =~ s/^(\w\w\w\w)(\w\w\w\w)(\w\w\w\w)$/$1\.$2\.$3/g;
	return $mac;
}

# sometimes you get a VERY VERY ill-behaved device
# which returns it's mac address table in a random order
# well, SNMP_Util requires that tables you walk be in increasing numerical order
# so a random order can cause your query to bail out half way through.
# the snmpwalk command has a flag to fix this, so here's some
# nasty-code to use it instead of SNMP_Util
sub ucd_snmpwalk
{
	my ($oid, $vlan) = (@_);
	my $ip = $$rh_session{ 'ip' };
	my $community = $$rh_session{ 'community' };
	if ( $vlan )
	{
		$community = "$community\@$vlan";
	}
	my $SNMPWALK_CMD =  "/usr/bin/snmpwalk -Cc -Ofn ";
	my %return;
	open(CMD, "$SNMPWALK_CMD $ip $community .$oid |" ) || die "Could not run command $SNMPWALK_CMD $ip $community $oid\n";
	while (my $line=<CMD>)
	{
		chomp($line);
		my ($roid, $val ) = split ( /=/, $line, 2);
		$val =~ s/^\s+//g;
		$roid =~ s/\s+$//g;
		$roid =~ s/^\.$oid\.//g;
		$return{ $roid } = $val;
	}
	return \%return;
}

# just anther helper function
# useful when printing large lists of vlans to the screen
# pass him an array of numbers and he'll join them with commas and dashes
# ex.
# 1,2,3,4,5,8,9,10,12,200,230
# becomes
# 1-5,8-9,10,12,200,230
sub consolidate_list
{
	my ($ra) = (@_);
	my $return;
	foreach my $number (sort {$a <=> $b} (@$ra) )
	{
		if ( $return eq "" ) { $return = ",$number"; next; }
		my $last_number = $return;
		$last_number =~ s/^.*[\,\-]([0-9]+)$/$1/g;
		if ( $number != $last_number + 1 )
		{
			$return = $return . ",$number";
		}
		else
		{
			# either replace the last number of a spread with this one
			$return =~ s/\-${last_number}$/-$number/g;
			# or append this one to the first number of a spread
			$return =~ s/\,${last_number}$/,${last_number}-$number/g;
		}
	}
	$return =~ s/^\,//g;
	return $return;
	
}


1;
__END__

=head1 NAME

coqm - Common OID Query Module

=head1 SYNOPSIS

	use coqm;
	my $rh_session = coqm::create_session($target_ip, $snmp_community_string );
	my $rh_ifDescr = coqm::fetch_ifDescr();
	
	foreach my $ifindex (sort { $a <=> $b } (keys %{$rh_ifDescr}) )
	{
		my $ifDescr = $$rh_ifDescr{ $ifindex };
		my $ifName = coqm::fetch_ifName( $ifindex );
		my $ifOperStatus = coqm::fetch_ifOperStatus( $ifindex );
		my $ifAdminStatus = coqm::fetch_ifAdminStatus( $ifindex );
		my $ifSpeed = coqm::fetch_ifSpeed( $ifindex );
		printf( "%3d %-20s %-20s %6s %6s %12s\n", $ifindex, $ifDescr, $ifName, $ifOperStatus, $ifAdminStatus, $ifSpeed );
	}
	
	

=head1 DESCRIPTION

coqm is a module for using SNMP to query commonly used OIDs on network devices.
The idea is to provide a consitent and streamlined interface for pulling information from the network devices rather than having the same OIDs and query routines buried in half a dozen different scripts.

NOTE:
Most of the functions in the coqm library follow this basic format:

if passed no values, they perform and SNMP walk and return a reference to hash whose keys are ifindices and whose values are the related values from the desired SNMP query (ie ifDescr, ifStatus, etc)

if passed an ifindex, they perform an SNMP get and return a scalar which is the value from the desired SNMP query (ie ifDescr, ifStatus, etc) for the passed ifindex.

ex.
my $rh_ifDescr = coqm::fetch_ifDescr();

return value is a reference to hash of ifindices and their ifDescr values.

my $ifDescr = coqm::fetch_ifDescr("1");

return value is a scalar containing the value of ifDescr for ifindex 1.



=head1 FUNCTIONS

=over 4

=item *
create_session( $ip, $community, $port, $timeout, $retries, $backoff, $version ) 

Info:
  Sets up a hash of parameters to be used for subsequent SNMP queries.  Returns a reference to this hash.

Argument(s):

  $ip

    The ip address (or valid DNS hostname) of the host to query.

  $community (optional)

    The SNMP community string to use when querying the host.

  $port (optional)

    The udp port to direct SNMP queries to.  Default value is 161 which is the SNMP default port.

  $timeout (optional)

    Time in seconds to wait for a response to an SNMP query.  Default value is 2.

  $retries (optional)

    Number of times to re-attempt a query if no response is recieved.  Default value is 3.

  $backoff (optional)

    Number of seconds to wait before retrying a query.  Default value is 1.

  $version (optional)

    SNMP version to use for queries.  Valid values are '1', '2', '2c'.  Default value is '1'.


Return value:

  returns a reference to hash whose keys are 'ip', 'community', 'port', etc. and whose values are the values for these parameters.

=item *
set_session($rh_sess) 

Info:
  Replaces the current hash of parameters for SNMP queires with the passed hash.  Subsequent queries will use the values stored in the passed hash.  Returns a reference to the displaced session hash.

  The general idea is to allow one to easily swap around between different query values (eg. host)

  Ex.
  my $rh_sess1 = coqm::create_session( $host1 );
  my $rh_sess2 = coqm::create_session( $host2 );
  for my $rh_sess ($rh_sess1, $rh_sess2)
  {
      coqm::set_session( $rh_sess );
      $rh_ifNames = coqm::fetch_ifName();
  }

  This snippet sets up a session hash for host1 and host2, and then queries the ifName values for each of them.
	

Argument(s):

  $rh_sess

    A reference to hash of the format returned by create_session().


Return value:

  returns a reference to the displaced session hash.

=item *
get_session()

Info:
  Returns a reference to the currently active session hash.  Included for completeness.

Return value:

  Returns a reference to the currently active session hash.

=item *
get($oid, $vlan) 

Info
  Performs and SNMP get on the passed oid using the session information from the most recent create_session() or set_session() call.

  A wrapper/replacement for the snmpget function of SNMP_util.pm.  The only real addition is the optional $vlan argument, which is used for performing 'community@vlan' style SNMP queries.

  For the unitiated, some vendor SNMP stacks (such as Cisco's) will return a different value for certain oids depending on what community string is used to query them.  Specifically, if the normal community string is appended with '@vlan' where 'vlan' is the vlan number of interest, the desired oid returns information as related to that vlan.  For example, on a Cisco device querying 'dot1dStpPortState' with the community string 'public@1' returns STP state information for vlan 1.  Querying 'dot1dStpPortState' with the community string 'public@20' returns STP state information for vlan 20.

Argument(s):

  $oid

    The SNMP oid to be queried.

  $vlan (optional)

    Vlan number for use in an 'community@vlan' style SNMP query (see Info).


Return value:

  returns a scalar value (the result of the desired SNMP get).

=item *
walk($oid, $vlan, $use_ucd_snmpwalk) 

Info:
  Performs an SNMP walk on the passed oid using the session information from the most recent create_session() or set_session() call.

  A wrapper around the snmpwalk function of SNMP_Util.pm.  Returns results as a hash of oid,value pairs.  Also handles 'community@vlan' style queries via the optional $vlan argument (see above discussion of get())

  Also contains an option to invoke the snmpwalk program from the UCD snmp package (now known as NET-SNMP) instead of the snmpwalk function of SNMP_util.pm to perform the desired query.  This option is present because SNMP_Util.pm cannot fully walk SNMP tables that appear out of strict numerical order.  One such table is 'dot1dTpFdbPort' which appears in arbitrary order on some SNMP stacks (ie Cisco 1900s).  The snmpwalk program from UCD has a flag to deal with these tables and so an option is provided for invoking this program to perform the walk.

Argument(s):

  $oid

    The SNMP oid to be walked.

  $vlan (optional)

    Vlan number for use in an 'community@vlan' style SNMP query (see above discussion for get()).

  $use_ucd_snmpwalk (optional)

    If set to anything, the external program snmpwalk (/usr/bin/snmpwalk) from the UCS snmp package (now called Net-SNMP) is used for the walk, instead of the snmpwalk function from SNMP_Util.pm (see discussion in Info section).


Return value:

  returns a reference to a hash whose keys are oids and whose values are the values for those oids from the desired SNMP walk.  For convience, only that portion of the SNMP oids beyond the passed oid is used for the keys of the returned hash, (ie oid.0.1 becomes hash key '0.1').

=item *
fetch_from_ifindex_table($oid, $ifindex, $ra_vals, $vlan) 

Info:
  Intended as an internal helper function for querying SNMP oid tables that are indexed by an ifindex (ie. ifDescr, ifSpeed, etc. etc.).
  If passed an ifindex, returns a scalar value, the result of calling get() get on $oid.$ifindex.
  If not passed an ifindex, returns a reference to hash which the result of calling walk() on oid.
  The general idea is that it either directly gives you the value for the ifindex you are looking for, or walks the entire table and returns it.
  Additionally, for SNMP tables that return a numeric value which corresponds to some text description as defined in the MIB (ex. ifAdminStatus), a reference to an array of such text values can be passed in and numeric values retrieved from the oid will be replaced by their corresponding values from this array.
  Finally, the optional $vlan argument provides a method for handling 'community@vlan' style queries see above discussion of get()).

Argument(s):

  $oid

    The oid of the SNMP table to be queired.  Should be singlely-indexed table such as ifDescr, ifSpeed, etc. etc.

  $ifindex (optional)

    The specific node of the oid table of interest.  For example, the ifindex to retrieve ifDescr for.

  $ra_vals (optional)

    Reference to an array.  Numeric values retrieved from the oid will be replaced by their corresponding values from this array.  Useful for SNMP tables such as ifAdminStatus which return a numeric value that corresponds to some text description defined the MIB.

  $vlan (optional)

    Vlan number for use in an 'community@vlan' style SNMP query (see above discussion for get()).


Return value:

  If passed an ifindex, returns a scalar value, the result of calling get() on $oid.$ifindex.
  If not passed an ifindex, returns a reference to a hash which the result of calling walk() on $oid.

=item *
fetch_ifDescr($ifindex)

Info:
  Retrieves value(s) from the ifDescr table.

Argument(s):

  $ifindex (optional)
   
    ifindex to retrieve ifDescr value for.

Return value:

  If passed an ifindex, returns a scalar value, the result of calling get() get on ifDescr.$ifindex.
  If not passed an ifindex, returns a reference to a hash which the result of calling walk() on ifDescr.

=item *
fetch_ifName($ifindex)

Info:
  Retrieves value(s) from the ifName table.

Argument(s):

  $ifindex (optional)
   
    ifindex to retrieve ifName value for.

Return value:

  If passed an ifindex, returns a scalar value, the result of calling get() get on ifName.$ifindex.
  If not passed an ifindex, returns a reference to a hash which the result of calling walk() on ifName.

=item *
fetch_ifOperStatus($ifindex)

Info:
  Retrieves value(s) from the ifOperStatus table.  Operating status of a port (line protocol up/down).

  The numeric values in the table are converted to their text values from the MIB.

  Those text values are: "up", "down", "testing", "unknown", "dormant", "notPresent", "lowerLayerDown"

Argument(s):

  $ifindex (optional)
   
    ifindex to retrieve ifOperStatus value for.

Return value:

  If passed an ifindex, returns a scalar value, the result of calling get() get on ifOperStatus.$ifindex.
  If not passed an ifindex, returns a reference to a hash which the result of calling walk() on ifOperStatus.

=item *
fetch_ifAdminStatus($ifindex)

Info:
  Retrieves value(s) from the ifAdminStatus table.  Administrative status of a port (administratively up/shutdown)

  The numeric values in the table are converted to their text values from the MIB.

  Those text values are: "up", "down", "testing"


Argument(s):

  $ifindex (optional)
   
    ifindex to retrieve ifAdminStatus value for.

Return value:

  If passed an ifindex, returns a scalar value, the result of calling get() get on ifAdminStatus.$ifindex.
  If not passed an ifindex, returns a reference to a hash which the result of calling walk() on ifAdminStatus.

=item *
fetch_ifSpeed($ifindex)

Info:
  Retrieves value(s) from the ifSpeed table.  Current operating speed of port in bit per second.

Argument(s):

  $ifindex (optional)
   
    ifindex to retrieve ifSpeed value for.

Return value:

  If passed an ifindex, returns a scalar value, the result of calling get() get on ifSpeed.$ifindex.
  If not passed an ifindex, returns a reference to a hash which the result of calling walk() on ifSpeed.

=item *
fetch_ifPhysAddress($ifindex)

Info:
  Retrieves value(s) from the ifPhysAddress table.

Argument(s):

  $ifindex (optional)
   
    ifindex to retrieve ifPhysAddress value for.

Return value:

  If passed an ifindex, returns a scalar value, the result of calling get() get on ifPhysAddress.$ifindex.
  If not passed an ifindex, returns a reference to a hash which the result of calling walk() on ifPhysAddress.

=item *
fetch_ifLastChange($ifindex)

Info:
  Retrieves value(s) from the ifLastChange table.

Argument(s):

  $ifindex (optional)
   
    ifindex to retrieve ifLastChange value for.

Return value:

  If passed an ifindex, returns a scalar value, the result of calling get() get on ifLastChange.$ifindex.
  If not passed an ifindex, returns a reference to a hash which the result of calling walk() on ifLastChange.


=item *
fetch_ifInOctets($ifindex)

Info:
  Retrieves value(s) from the ifInOctets table.

Argument(s):

  $ifindex (optional)
   
    ifindex to retrieve ifInOctets value for.

Return value:

  If passed an ifindex, returns a scalar value, the result of calling get() get on ifInOctets.$ifindex.
  If not passed an ifindex, returns a reference to a hash which the result of calling walk() on ifInOctets.

=item *
fetch_ifInUcastPkts($ifindex)

Info:
  Retrieves value(s) from the ifInUcastPkts table.

Argument(s):

  $ifindex (optional)
   
    ifindex to retrieve ifInUcastPkts value for.

Return value:

  If passed an ifindex, returns a scalar value, the result of calling get() get on ifInUcastPkts.$ifindex.
  If not passed an ifindex, returns a reference to a hash which the result of calling walk() on ifInUcastPkts.

=item *
fetch_ifInNUcastPkts($ifindex)

Info:
  Retrieves value(s) from the ifInNUcastPkts table.

Argument(s):

  $ifindex (optional)
   
    ifindex to retrieve ifInNUcastPkts value for.

Return value:

  If passed an ifindex, returns a scalar value, the result of calling get() get on ifInNUcastPkts.$ifindex.
  If not passed an ifindex, returns a reference to a hash which the result of calling walk() on ifInNUcastPkts.

=item *
fetch_ifInDiscards($ifindex)

Info:
  Retrieves value(s) from the ifInDiscards table.

Argument(s):

  $ifindex (optional)
   
    ifindex to retrieve ifInDiscards value for.

Return value:

  If passed an ifindex, returns a scalar value, the result of calling get() get on ifInDiscards.$ifindex.
  If not passed an ifindex, returns a reference to a hash which the result of calling walk() on ifInDiscards.

=item *
fetch_ifInErrors($ifindex)

Info:
  Retrieves value(s) from the ifInErrors table.

Argument(s):

  $ifindex (optional)
   
    ifindex to retrieve ifInErrors value for.

Return value:

  If passed an ifindex, returns a scalar value, the result of calling get() get on ifInErrors.$ifindex.
  If not passed an ifindex, returns a reference to a hash which the result of calling walk() on ifInErrors.

=item *
fetch_ifInUnknownProtos($ifindex)

Info:
  Retrieves value(s) from the ifInUnknownProtos table.

Argument(s):

  $ifindex (optional)
   
    ifindex to retrieve ifInUnknownProtos value for.

Return value:

  If passed an ifindex, returns a scalar value, the result of calling get() get on ifInUnknownProtos.$ifindex.
  If not passed an ifindex, returns a reference to a hash which the result of calling walk() on ifInUnknownProtos.

=item *
fetch_ifOutOctets($ifindex)

Info:
  Retrieves value(s) from the ifOutOctets table.

Argument(s):

  $ifindex (optional)
   
    ifindex to retrieve ifOutOctets value for.

Return value:

  If passed an ifindex, returns a scalar value, the result of calling get() get on ifOutOctets.$ifindex.
  If not passed an ifindex, returns a reference to a hash which the result of calling walk() on ifOutOctets.

=item *
fetch_ifOutUcastPkts($ifindex)

Info:
  Retrieves value(s) from the ifOutUcastPkts table.

Argument(s):

  $ifindex (optional)
   
    ifindex to retrieve ifOutUcastPkts value for.

Return value:

  If passed an ifindex, returns a scalar value, the result of calling get() get on ifOutUcastPkts.$ifindex.
  If not passed an ifindex, returns a reference to a hash which the result of calling walk() on ifOutUcastPkts.

=item *
fetch_ifOutNUcastPkts($ifindex)

Info:
  Retrieves value(s) from the ifOutNUcastPkts table.

Argument(s):

  $ifindex (optional)
   
    ifindex to retrieve ifOutNUcastPkts value for.

Return value:

  If passed an ifindex, returns a scalar value, the result of calling get() get on ifOutNUcastPkts.$ifindex.
  If not passed an ifindex, returns a reference to a hash which the result of calling walk() on ifOutNUcastPkts.

=item *
fetch_ifOutDiscards($ifindex)

Info:
  Retrieves value(s) from the ifOutDiscards table.

Argument(s):

  $ifindex (optional)
   
    ifindex to retrieve ifOutDiscards value for.

Return value:

  If passed an ifindex, returns a scalar value, the result of calling get() get on ifOutDiscards.$ifindex.
  If not passed an ifindex, returns a reference to a hash which the result of calling walk() on ifOutDiscards.

=item *
fetch_ifOutErrors($ifindex)

Info:
  Retrieves value(s) from the ifOutErrors table.

Argument(s):

  $ifindex (optional)
   
    ifindex to retrieve ifOutErrors value for.

Return value:

  If passed an ifindex, returns a scalar value, the result of calling get() get on ifOutErrors.$ifindex.
  If not passed an ifindex, returns a reference to a hash which the result of calling walk() on ifOutErrors.

=item *
fetch_ifOutQLen($ifindex)

Info:
  Retrieves value(s) from the ifOutQLen table.

Argument(s):

  $ifindex (optional)
   
    ifindex to retrieve ifOutQLen value for.

Return value:

  If passed an ifindex, returns a scalar value, the result of calling get() get on ifOutQLen.$ifindex.
  If not passed an ifindex, returns a reference to a hash which the result of calling walk() on ifOutQLen.

=item *
fetch_ifSpecific($ifindex)

Info:
  Retrieves value(s) from the ifSpecific table.

Argument(s):

  $ifindex (optional)
   
    ifindex to retrieve ifSpecific value for.

Return value:

  If passed an ifindex, returns a scalar value, the result of calling get() get on ifSpecific.$ifindex.
  If not passed an ifindex, returns a reference to a hash which the result of calling walk() on ifSpecific.

=item *
fetch_sysClearPortTime($ifindex) 

Info:
  Retrieves value(s) from the sysClearPortTime table.  Gives hundredths of a second since port counters were cleared.

Argument(s):

  $ifindex (optional)

    ifindex to retrieve sysClearPortTime value for.


Return value:

  If passed an ifindex, returns a scalar value, the result of calling get() get on sysClearPortTime.$ifindex.
  If not passed an ifindex, returns a reference to a hash which the result of calling walk() on sysClearPortTime.

=item *
fetch_ifAccessVlan($ifindex)

Info:
  Retrieves value(s) from the vmVlan (Access Vlan) table.

Argument(s):

  $ifindex (optional)
   
    ifindex to retrieve vmVlan value for.

Return value:

  If passed an ifindex, returns a scalar value, the result of calling get() get on vmVlan.$ifindex.
  If not passed an ifindex, returns a reference to a hash which the result of calling walk() on vmVlan.


=item *
ParseVlanBitPattern($VlanBitPattern) 

Info:
  Helper function.  Converts a binary value of the type returned by querying the vlanTrunkPortVlansEnabled oid into a list of vlan numbers.

Argument(s):

  $VlanBitPattern

    Binary value, each bit corresponds to a vlan, 0 means vlan disabled, 1 means vlan enabled.


Return value:

  returns an array of vlan numbers (all enabled vlans).

=item *
fetch_vlanTrunkPortVlansEnabled($ifindex) 

Info:
  The vlanTrunkPortVlansEnabled out of Cisco's VTP MIB returns a binary value each bit of which represents the configuration of trunking for a given vlan on a particular ifindex.  If the bit is turned on ('1') then trunking is enabled for that vlan, if it's off ('0') then trunking is disabled.

  fetch_vlanTrunkPortVlansEnabled() retrieves this value for an ifindex (or for all ifindices if not passed an ifindex) and converts it into an array containing all the vlan numbers marked as enabled.

  Returns a reference to this array (or a hash of references to arrays if called w/o an ifindex).

  Effectively, this is similar to looking at the 'vlans allowed' portion of a 'show trunk' or 'show interface ... trunk' command.


Argument(s):

  $ifindex (optional)

    ifindex to retrieve vlanTrunkPortVlansEnabled for.


Return value:

  If passed an ifindex, returns a scalar value, the reference to an array of vlans enabled for trunking on the passed ifindex.
  If not passed an ifindex, returns a reference to a hash whose keys are ifindices and whose values are references to arrays of vlans enabled for trunking on those ifindices.


=item *
fetch_vlanTrunkPortNativeVlan($ifindex)

Info:
  Retrieves value(s) from the vlanTrunkPortNativeVlan table.

Argument(s):

  $ifindex (optional)
   
    ifindex to retrieve vlanTrunkPortNativeVlan value for.

Return value:

  If passed an ifindex, returns a scalar value, the result of calling get() get on vlanTrunkPortNativeVlan.$ifindex.
  If not passed an ifindex, returns a reference to a hash which the result of calling walk() on vlanTrunkPortNativeVlan.

=item *
fetch_vlanTrunkPortDynamicStatus($ifindex)

Info:
  Retrieves value(s) from the vlanTrunkPortDynamicStatus table.  Tells how a port is operating with regards to trunking.

  The numeric values in the table are converted to their text values from the MIB.

  Those text values are: "trunk", "access"

  "trunk" indicates that the port is acting as a trunk port.

  "access" indicates that the port is acting as an access port.

Argument(s):

  $ifindex (optional)
   
    ifindex to retrieve vlanTrunkPortDynamicStatus value for.

Return value:

  If passed an ifindex, returns a scalar value, the result of calling get() get on vlanTrunkPortDynamicStatus.$ifindex.
  If not passed an ifindex, returns a reference to a hash which the result of calling walk() on vlanTrunkPortDynamicStatus.

=item *
fetch_vlanTrunkPortDynamicState($ifindex)

Info:
  Retrieves value(s) from the vlanTrunkPortDynamicState table.  Tells how a port is configured with regards to trunking.

  The numeric values in the table are converted to their text values from the MIB.

  Those text values are: "on", "off", "desirable", "auto", "onNoNegotiate"

Argument(s):

  $ifindex (optional)
   
    ifindex to retrieve vlanTrunkPortDynamicState value for.

Return value:

  If passed an ifindex, returns a scalar value, the result of calling get() get on vlanTrunkPortDynamicState.$ifindex.
  If not passed an ifindex, returns a reference to a hash which the result of calling walk() on vlanTrunkPortDynamicState.

=item *
fetch_vlanTrunkPortEncapsulationType($ifindex)

Info:
  Retrieves value(s) from the vlanTrunkPortEncapsulationType table.  Tells how the port is configured with regards to vlan trunk encapsulation.

  The numeric values in the table are converted to their text values from the MIB.

  Those text values are: "isl", "dot10", "lane", "dot1Q", "negotiate"

Argument(s):

  $ifindex (optional)
   
    ifindex to retrieve vlanTrunkPortEncapsulationType value for.

Return value:

  If passed an ifindex, returns a scalar value, the result of calling get() get on vlanTrunkPortEncapsulationType.$ifindex.
  If not passed an ifindex, returns a reference to a hash which the result of calling walk() on vlanTrunkPortEncapsulationType.

=item *
fetch_vlanTrunkPortEncapsulationOperType($ifindex)

Info:
  Retrieves value(s) from the vlanTrunkPortEncapsulationOperType table.  Tells how the port is operating with regards to vlan trunk encapsulation.

  The numeric values in the table are converted to their text values from the MIB.

  Those text values are: "isl", "dot10", "lane", "dot1Q", "negotiating", "notApplicable"


Argument(s):

  $ifindex (optional)
   
    ifindex to retrieve vlanTrunkPortEncapsulationOperType value for.

Return value:

  If passed an ifindex, returns a scalar value, the result of calling get() get on vlanTrunkPortEncapsulationOperType.$ifindex.
  If not passed an ifindex, returns a reference to a hash which the result of calling walk() on vlanTrunkPortEncapsulationOperType.



=item *
fetch_HCoid($oid, $ifindex) 

Info:
  Helper function.  The HC or 'High Counter' oids can only be queried with SNMP v2 or above, so this function just makes sure the current session is set to use SNMP 2c queries (or better) before calling fetch_from_ifindex_table() to execute the query.  After the query is executed, the previous SNMP version of the current session is restored.

Argument(s):

  $oid

    oid to be queried.

  $ifindex (optional)

    ifindex to be queried.


Return value:

  returns the results of fetch_from_ifindex_table() for the passed $oid and $ifindex.

=item *
fetch_ifHCInOctets($ifindex)

Info:
  Retrieves value(s) from the ifHCInOctets table.

Argument(s):

  $ifindex (optional)
   
    ifindex to retrieve ifHCInOctets value for.

Return value:

  If passed an ifindex, returns a scalar value, the result of calling get() get on ifHCInOctets.$ifindex.
  If not passed an ifindex, returns a reference to a hash which the result of calling walk() on ifHCInOctets.

=item *
fetch_ifHCInUcastPkts($ifindex)

Info:
  Retrieves value(s) from the ifHCInUcastPkts table.

Argument(s):

  $ifindex (optional)
   
    ifindex to retrieve ifHCInUcastPkts value for.

Return value:

  If passed an ifindex, returns a scalar value, the result of calling get() get on ifHCInUcastPkts.$ifindex.
  If not passed an ifindex, returns a reference to a hash which the result of calling walk() on ifHCInUcastPkts.

=item *
fetch_ifHCInMulticastPkts($ifindex)

Info:
  Retrieves value(s) from the ifHCInMulticastPkts table.

Argument(s):

  $ifindex (optional)
   
    ifindex to retrieve ifHCInMulticastPkts value for.

Return value:

  If passed an ifindex, returns a scalar value, the result of calling get() get on ifHCInMulticastPkts.$ifindex.
  If not passed an ifindex, returns a reference to a hash which the result of calling walk() on ifHCInMulticastPkts.

=item *
fetch_ifHCInBroadcastPkts($ifindex)

Info:
  Retrieves value(s) from the ifHCInBroadcastPkts table.

Argument(s):

  $ifindex (optional)
   
    ifindex to retrieve ifHCInBroadcastPkts value for.

Return value:

  If passed an ifindex, returns a scalar value, the result of calling get() get on ifHCInBroadcastPkts.$ifindex.
  If not passed an ifindex, returns a reference to a hash which the result of calling walk() on ifHCInBroadcastPkts.

=item *
fetch_ifHCOutOctets($ifindex)

Info:
  Retrieves value(s) from the ifHCOutOctets table.

Argument(s):

  $ifindex (optional)
   
    ifindex to retrieve ifHCOutOctets value for.

Return value:

  If passed an ifindex, returns a scalar value, the result of calling get() get on ifHCOutOctets.$ifindex.
  If not passed an ifindex, returns a reference to a hash which the result of calling walk() on ifHCOutOctets.

=item *
fetch_ifHCOutUcastPkts($ifindex)

Info:
  Retrieves value(s) from the ifHCOutUcastPkts table.

Argument(s):

  $ifindex (optional)
   
    ifindex to retrieve ifHCOutUcastPkts value for.

Return value:

  If passed an ifindex, returns a scalar value, the result of calling get() get on ifHCOutUcastPkts.$ifindex.
  If not passed an ifindex, returns a reference to a hash which the result of calling walk() on ifHCOutUcastPkts.

=item *
fetch_ifHCOutMulticastPkts($ifindex)

Info:
  Retrieves value(s) from the ifHCOutMulticastPkts table.

Argument(s):

  $ifindex (optional)
   
    ifindex to retrieve ifHCOutMulticastPkts value for.

Return value:

  If passed an ifindex, returns a scalar value, the result of calling get() get on ifHCOutMulticastPkts.$ifindex.
  If not passed an ifindex, returns a reference to a hash which the result of calling walk() on ifHCOutMulticastPkts.

=item *
fetch_ifHCOutBroadcastPkts($ifindex)

Info:
  Retrieves value(s) from the ifHCOutBroadcastPkts table.

Argument(s):

  $ifindex (optional)
   
    ifindex to retrieve ifHCOutBroadcastPkts value for.

Return value:

  If passed an ifindex, returns a scalar value, the result of calling get() get on ifHCOutBroadcastPkts.$ifindex.
  If not passed an ifindex, returns a reference to a hash which the result of calling walk() on ifHCOutBroadcastPkts.

=item *
fetch_cdpGlobalDeviceId()

Info:
  Retrieves the value from cdpGlobalDeviceId for the host.  This value holds the deviceid ('set system name ...' in CatOS, 'hostname ...' in IOS) of the target host.  Not be confused with a proper DNS hostname, the deviceid is just whatever the host has been told it's name is.

Return value:

  returns a scalar, the result of calling get() on cdpGlobalDeviceId.

=item *
fetch_from_cdp_tree($oid, $ifindex, $pid) 

Info:
  Helper function.  This one walks or gets from the doubly indexed cdpCache family of oids.  These tables are first indexed by ifindex then by 'cdp process id' (pid for short in this code) which is just an arbitrary number used to allow multiple cdp neighbors to be returned for the same ifindex.

Argument(s):

  $oid

    The SNMP OID from the cdpCache family (ie cdpCacheAddress, etc) to be queried.

  $ifindex (optional)

    ifindex to retrieve the passed oid for.

  $pid (optional)

    cdp process id to  retrieve the passed oid for.


Return value:

  If passed just an oid, returns a reference to a hash whose keys are strings of the form 'ifindex.pid' (ie. '2.1') and whose values are the oid's value for the corresponding ifindex and cdp process id (pid).

  If passed oid and an ifindex, returns a reference to a hash whose keys are cdp process ids and whose values are the oid's values for the ifindex and cdp process id.

  If passed an oid and an ifindex and a cdp process id, returns a scalar whose value is the oid's value for passed ifindex and cdp process id.

  

=item *
safe_inet_ntoa($addr) 

Info:
  Just a quick and dirty helper that checks the length of the passed value and returns "NULL" if it is less than 4 bytes, otherwise returns the value of inet_ntoa($addr).  Used in the fetch_cdpCacheAddress() function to avoid a program blow up from calling inet_ntoa() on an empty/malformed binary ip address (occassionally the SNMP stack will come back with these).

Argument(s):

  $addr

    IP address in binary format to convert to ASCII representation.


Return value:

  If len($addr) < 4, returns "NULL"

  Else returns inet_ntoa($addr)

=item *
fetch_cdpCacheAddress($ifindex, $pid) 

Info:
  Retrieves values from cdpCacheAddress oid.  This is a Cisco specific OID that returns the IP Address (in binary format) of a host's CDP neighbors.  This function converts these binary IP addresses into ASCII form (ie. 192.168.1.1).


Argument(s):

  $ifindex (optional)

    ifindex for which to retrieve cdpCacheAddress for.

  $pid (optional)

    cdp process id for which to retrieve cdpCacheAddress for.
    The 'cdp process id' is just and arbitrary number which is used to distinguish between multiple cdp neighbors appearing on the same ifindex.


Return value:

  If passed no arguments, returns a reference to a hash whose keys are strings of the form 'ifindex.pid' (ie. '2.1') and whose values are the cdpCacheAddress value for the corresponding ifindex and cdp process id (pid).

  If passed oid and an ifindex, returns a reference to a hash whose keys are cdp process ids and whose values are the cdpCacheAddress values for the ifindex and cdp process id.

  If passed an oid and an ifindex and a cdp process id, returns a scalar whose value is the cdpCacheAddress value for passed ifindex and cdp process id.

  All returned values for cdpCacheAddress are converted from binary to ASCII format with inet_ntoa().

=item *
fetch_cdpCacheDeviceId($ifindex, $pid) 

Info:
  Retrieves values from cdpCacheDeviceId oid.  This is a Cisco specific OID that returns the 'DeviceId' of device's CDP neighbors.  See fetch_cdpGlobalDeviceId() for a discussion of what a 'DeviceId' is.


Argument(s):

  $ifindex (optional)

    ifindex for which to retrieve cdpCacheDeviceId for.

  $pid (optional)

    cdp process id for which to retrieve cdpCacheDeviceId for.
    The 'cdp process id' is just and arbitrary number which is used to distinguish between multiple cdp neighbors appearing on the same ifindex.


Return value:

  If passed no arguments, returns a reference to a hash whose keys are strings of the form 'ifindex.pid' (ie. '2.1') and whose values are the cdpCacheDeviceId value for the corresponding ifindex and cdp process id (pid).

  If passed oid and an ifindex, returns a reference to a hash whose keys are cdp process ids and whose values are the cdpCacheDeviceId values for the ifindex and cdp process id.

  If passed an oid and an ifindex and a cdp process id, returns a scalar whose value is the cdpCacheDeviceId value for passed ifindex and cdp process id.


=item *
fetch_cdpCacheDevicePort($ifindex, $pid) 

Info:
  Retrieves values from cdpCacheDevicePort oid.  This is a Cisco specific OID that returns a text description of the remote ports for a host's CDP neighbors.  That is, for a given cdp neighbor it gives a description (ie. Gi2/1) of the neighbor's port.


Argument(s):

  $ifindex (optional)

    ifindex for which to retrieve cdpCacheDevicePort for.

  $pid (optional)

    cdp process id for which to retrieve cdpCacheDevicePort for.
    The 'cdp process id' is just and arbitrary number which is used to distinguish between multiple cdp neighbors appearing on the same ifindex.


Return value:

  If passed no arguments, returns a reference to a hash whose keys are strings of the form 'ifindex.pid' (ie. '2.1') and whose values are the cdpCacheDevicePort value for the corresponding ifindex and cdp process id (pid).

  If passed oid and an ifindex, returns a reference to a hash whose keys are cdp process ids and whose values are the cdpCacheDevicePort values for the ifindex and cdp process id.

  If passed an oid and an ifindex and a cdp process id, returns a scalar whose value is the cdpCacheDevicePort value for passed ifindex and cdp process id.


=item *
fetch_cdpCachePlatform($ifindex, $pid) 

Info:
  Retrieves values from cdpCachePlatform oid.  This is a Cisco specific OID that returns a text description of the model type (ie WS-C6509, WS-C3550-24-PWR, etc.) of a host's CDP neighbors.


Argument(s):

  $ifindex (optional)

    ifindex for which to retrieve cdpCacheDevicePlatform for.

  $pid (optional)

    cdp process id for which to retrieve cdpCacheDevicePlatform for.
    The 'cdp process id' is just and arbitrary number which is used to distinguish between multiple cdp neighbors appearing on the same ifindex.


Return value:

  If passed no arguments, returns a reference to a hash whose keys are strings of the form 'ifindex.pid' (ie. '2.1') and whose values are the cdpCacheDevicePlatform value for the corresponding ifindex and cdp process id (pid).

  If passed oid and an ifindex, returns a reference to a hash whose keys are cdp process ids and whose values are the cdpCacheDevicePlatform values for the ifindex and cdp process id.

  If passed an oid and an ifindex and a cdp process id, returns a scalar whose value is the cdpCacheDevicePlatform value for passed ifindex and cdp process id.


=item *
fetch_cdpCacheNativeVLAN($ifindex, $pid) 

Info:
  Retrieves values from cdpCacheNativeVLAN oid.  This is a Cisco specific OID that returns the 'Native VLAN' as configured on neighbor's port for a host's CDP neighbors.  When compared with the host's own native vlan on a port (vlanTrunkPortNativeVlan), useful for checking for 'native vlan mismatches'.


Argument(s):

  $ifindex (optional)

    ifindex for which to retrieve cdpCacheDeviceNativeVLAN for.

  $pid (optional)

    cdp process id for which to retrieve cdpCacheDeviceNativeVLAN for.
    The 'cdp process id' is just and arbitrary number which is used to distinguish between multiple cdp neighbors appearing on the same ifindex.


Return value:

  If passed no arguments, returns a reference to a hash whose keys are strings of the form 'ifindex.pid' (ie. '2.1') and whose values are the cdpCacheDeviceNativeVLAN value for the corresponding ifindex and cdp process id (pid).

  If passed oid and an ifindex, returns a reference to a hash whose keys are cdp process ids and whose values are the cdpCacheDeviceNativeVLAN values for the ifindex and cdp process id.

  If passed an oid and an ifindex and a cdp process id, returns a scalar whose value is the cdpCacheDeviceNativeVLAN value for passed ifindex and cdp process id.


=item *
fetch_cdpCacheDuplex($ifindex, $pid) 

Info:
  Retrieves values from cdpCacheDuplex oid.  This is a Cisco specific OID that returns the duplex (half/full) value of the remote ports for a host's CDP neighbors.  As an example, if port A on Switch1 connects to port B on Switch2, querying cdpCacheDuplex on Switch1 would give you the duplex setting of port B on switch2.  This is a useful OID for checking for duplex mismatches.


Argument(s):

  $ifindex (optional)

    ifindex for which to retrieve cdpCacheDuplex for.

  $pid (optional)

    cdp process id for which to retrieve cdpCacheDuplex for.
    The 'cdp process id' is just and arbitrary number which is used to distinguish between multiple cdp neighbors appearing on the same ifindex.


Return value:

  If passed no arguments, returns a reference to a hash whose keys are strings of the form 'ifindex.pid' (ie. '2.1') and whose values are the cdpCacheDuplex value for the corresponding ifindex and cdp process id (pid).

  If passed oid and an ifindex, returns a reference to a hash whose keys are cdp process ids and whose values are the cdpCacheDuplex values for the ifindex and cdp process id.

  If passed an oid and an ifindex and a cdp process id, returns a scalar whose value is the cdpCacheDuplex value for passed ifindex and cdp process id.


=item *
fetch_vlans()

Info:
   Attempts to cull a list of all the vlans known to a host.  Currently this is done by walking vtpVlanName which is a Cisco specific oid, utlimately would like to add support retrieving the available vlans on other device types.

Return value:

  returns a reference to an array whose values are the vlan numbers known to the queried host.
  A copy of this reference is also inserted into the current session hash under the key 'vlans'.

=item *
setup_vlans_array($vlan) 

Info:
  A helper function which either constructs an array of the single passed value.  If passed no value, the function looks in the current session hash for the key 'vlans'.  If this key exists, it is assumed to contain a reference to an array of all the vlans available to a host as discovered via a previous call to fetch_vlans(). 
   If the 'vlans' key does not exists, the function fetch_vlans() is called and it's return value is used as a reference to an array of all the vlans available to the host.

  Again, this is just a helper function used to help aide in the reuse of certain code loops in the coqm module.  It is only documented here for completeness.

Argument(s):

  $vlan (optional)

    vlan number.


Return value:

  returns a reference to an array which contains either just the passed vlan, or all the vlans available on the host if no vlan is passed in.

=item *
fetch_bridgeport_to_ifindex($bridgeport, $vlan) 

Info:
  Walks or gets from the dot1dBasePortIfIndex table.  This table maps 'bridgeports' to 'ifindices'.  Functionally the two are the same thing, they both represent ports on a host, but (for whatever reason) they are different and thus it is necessary to have a table which maps between the two.  The need is particularly strong because there is no table that maps bridgeports to text descriptions, so to actually know what port on a device a bridgeport you've got to look up it's ifindex mapping in dot1dBasePortIfIndex and then look up that ifindex's description/name in ifDescr/ifName.

  The dot1dBasePortIfIndex table is one which some SNMP stacks (aka Cisco's) require one to query with the 'community@vlan' syntax (see discussion for get() above), hence the $vlan argument.

  For those who'd rather pretend this query syntax didn't exist, fetch_bridgeport_to_ifindex() can be called w/o the optional $vlan argument, in which case it will automatically retrieve all the vlans on a switch (via fetch_vlans()) and walk/get from fetch_bridgeport_to_ifindex() using them.  The only cost to doing things this way is speed.  Also be advised that if one wishes to retrieve the complete table of bridgeport to ifindex mappings the $vlan argument should be omitted because otherwise one will only retrieve to mapping for ports in the passed vlan!

Argument(s):

  $bridgeport (optional)

    The specific bridgeport to retrieve an ifindex mapping for.

  $vlan (optional)

    The vlan to query the dot1dBasePortIfIndex for.


Return value:

  If called with no arguments, returns a reference to hash whose keys are brigdeports and whose values are ifindices.  This hash will contain the host's complete mapping from bridgeport to ifindices, although it should be noted that in some rare cases this will not include all the ifindices and/or bridgeports on a device, since a complete bridgeport to ifindex and vice versa mapping does not always exist (think wierd Cisco ATM LANE boards).

  If called with a bridgeport, returns a scalar whose value is the ifindex mapping for that bridgeport.

  If called with a vlan, only queries dot1dBasePortIfIndex with the community string 'community@vlan', where community is the SNMP community string as setup by the most recent create_session() or set_session() call, and vlan is the passed vlan number.


=item *
fetch_dot1dStpPortState( $bridgeport, $ra_vlans ) 

Info:
  Queries the dot1dStpPortState table.  This Cisco-specific oid table contains the STP (Spanning-Tree Protocol) state for all the vlans and ports on a device. 

  The table is indexed by bridgeport and it is queried using the 'community@vlan' syntax (see discussion for get() above).  Essentially, the table must be queried seperately for each vlan to retrieve the STP status for that vlan.

  For those who would prefer to ignore this syntax, fetch_dot1dStpPortState() can be called w/o the $ra_vlans argument, in which case it will automatically retrieve all the vlans on a switch (via fetch_vlans()) and query dot1dStpPortState for their STP forwarding state.

  The numeric values in this table are converted into their text values from the MIB.

  Those text values are:  "NULL", "disabled", "blocking", "listening", "learning", "forwarding", "broken"

Argument(s):

  $bridgeport (optional)

    The brideport number of the host's port to retrieve dot1dStpPortState value(s) for.

  $ra_vlans (optional)

    Either a scalar, indicating a single vlan to retrieve a dot1dStpPortState value for.  Or a reference to an array of vlans to retrieve dot1dStpPortState values for.  One way to retrieve such an array might be via a call to fetch_vlanTrunkPortVlansEnabled().

    To prevent timeouts from querying the host at an invalid community string, any passed vlan or vlans are vetted against the list of all vlans available on teh host as discovered via fetch_vlans().


Return value:

  if called with no arguments, returns a reference to a doubly indexed hash whose keys are vlans and whose values are references to a hash whose keys are bridgeports and whose values are dot1dStpPortState table values for the given vlan on the given bridgeport.  In code terms this is a hash like so: $$rh{ $vlan }{ $bridgeport } = $StpState

  if called with a just bridgeport value, returns a reference to a hash whose keys are vlans and whose values are dot1dStpPortState table values for the given vlan on the given bridgeport.  In code terms this is a hash like so: $$rh{ $vlan } = $StpState


  if called with a just a single vlan value, returns a reference to a hash whose keys are bridgeports and whose values are dot1dStpPortState table values for the given vlan on the given bridgeport.  In code terms this is a hash like so: $$rh{ $bridgeport } = $StpState

  if called with just a reference to an array of vlans, returns a reference to a doubly indexed hash whose keys are vlans and whose values are references to a hash whose keys are bridgeports and whose values are dot1dStpPortState table values for the given vlan on the given bridgeport.  In code terms this is a hash like so: $$rh{ $vlan }{ $bridgeport } = $StpState

  if called with a bridgeport value and a reference to an array of vlans, returns a reference to a hash whose keys are vlans and whose values are dot1dStpPortState table values for the given vlan on the given bridgeport.  In code terms this is a hash like so: $$rh{ $vlan } = $StpState

  if called with bridgeport and a single vlan value, returns a scalar whose value is the dot1dStpPortState table values for the given vlan on the given bridgeport.


=item *
fetch_mac_bridgeport_and_vlan($mac, $vlan) 

Info:
  Queries the dot1dTpFdbPort table to retrieve the bridgeport number of the port associated with a mac address in the device's forwarding/CAM table.  Will retrieve the entire cam table if no mac address value is passed in.

  The dot1dTpFdbPort is indexed by mac address as converted into an OID.  fetch_mac_bridgeport_and_vlan() handles the conversion of mac addresses from the Cisco IOS style (ex. 0004.ac7c.a766) into OID form.  And it also handles performing this converstion in reverse for it's return value(s).

  The dot1dTpFdbPort is also is queried using the 'community@vlan' syntax (see discussion for get() above).  Essentially, the table must be queried seperately for each vlan to retrieve CAM/forwarding table entries for each vlan.

  For those who'd like to ignore this syntax, fetch_mac_bridgeport_and_vlan() can be called w/o the optional vlan argument, in which case it will automatically retrieve all the vlans on a switch (via fetch_vlans()) and query dot1dTpFdbPort for each one until it finds the value(s) it is looking for.  The only penalty for doing things this way is speed.

Argument(s):

  $mac (optional)

    Mac address to query dot1dTpFdbPort for, in Cisco IOS style (ex. 0004.ac7c.a766).

  $vlan (optional)

    The vlan to query dot1dTpFdbPort for.


Return value:

  If called with a mac address value, returns a reference to a hash whose keys are 'vlan' and 'bridgeport' and whose values are the vlan and bridgeport for the passed mac address as found in the devices CAM/forwarding table.

  If passed no mac address value, returns a reference to a doubly indexed hash whose keys are 'bridgeport', 'vlan', and 'by_bridgeport'.

  The value for the key 'bridgeport' is a reference to a hash whose keys are mac addresses and whose values are bridgeports for those mac addresses as found in the devices CAM/forwarding table.  In code terms this looks like: $$rh{ 'bridgeports' }{ $mac } = $bridgeport

  The value for the key 'vlan' is a reference to a hash whose keys are mac addresses and whose values are vlans for those mac addresses as found in the devices CAM/forwarding table.  In code terms this looks like: $$rh{ 'vlans' }{ $mac } = $bridgeport

  The value for the key 'by_bridgeport' is a reference to a hash whose keys are bridgeport numbers and whose values are references to an array of mac addresses associated with that bridgeport via the devices' CAM/forwarding table.  In code terms this looks like: $$rh{ 'by_bridgeport' }{ $bridgeport } = \@mac_addresses

  Notice:  By indexing these hashes by mac address, this approach leaves out the possibility of the same mac address appearing on two different ports/vlans on the same switch (think Solaris machines that straddle two subnets).  Presently there is no fix for this, though one is in the works (basically a few extra keys will be added to the return hash.)

  If passed a vlan but no mac address, return format is the same as if passed no arguments, but only CAM/forwarding table entries for the passed vlan are included in the output.

  If one is wondering why the vlan information is included in this function at all, the simple answer is speed.  Querying a device once per vlan can be a real drag, esp. on a larger device with more vlans.  If one knows the vlan a mac address lives on beforehand, this process can be skipped.  Hence this function makes sure to accept and return vlan information about the mac address, so that either the current or future queries can be sped up.


=item *
fetch_cisco_cam_table()

Info:
  A wrapper function to retrieve the CAM/mac address forwarding table from a Cisco switch or any other device that responds to the dot1dTpFdbPort SNMP OID table.  The function does a double job of both pulling the dot1dTpFdbPort table (which yields bridgeport to mac address mappings) from the device and pulling the dot1dBasePortIfIndex table so that it can return a mapping from mac addresses to ifindices and vice versa.  This is considered desireable since ifindices are the more commonly used port identifier in SNMP and ifDescr/ifName exist to map ifindices to text descriptions of a port (ie. Gi2/1) but no such table exists to map bridgeports to text descriptions.

Return value:

  returns a reference to a hash whose keys are 'by_ifindex' and 'by_mac'.

  The value for 'by_mac' is a reference to a hash whose keys are mac addresses and whose values are the ifindices for those mac addresses as held in the device's CAM/forwarding table.  In code terms this looks like: $$rh{ 'by_mac' }{ $mac } = $ifindex

  The value for 'by_ifindex' is a reference to a hash whose keys are ifindices and whose values are references to an array of mac addresses associated with the ifinices in the device's CAM/forwarding table.  In code terms this looks like:  $$rh{ 'by_ifindex' }{ $ifindex } = \@macs

=item *
fetch_extreme_cam_table()

Info:
  Switches/Routers made by Extreme Networks use the SNMP OID "1.3.6.1.4.1.1916.1.16.1.1.3" for querying their forwarding/CAM table information, although they also optionally support responding to the dot1dTpFdbPort table as well.  This function walks the table found at "1.3.6.1.4.1.1916.1.16.1.1.3" to collect CAM/forwarding table information from an Extreme Networks device.

Return value:

  returns a reference to a hash whose keys are 'by_mac' and 'by_ifindex'.

  The value for 'by_mac' is a reference to a hash whose keys are mac addresses and whose values are the ifindices for those mac addresses as held in the device's CAM/forwarding table.  In code terms this looks like: $$rh{ 'by_mac' }{ $mac } = $ifindex

  The value for 'by_ifindex' is a reference to a hash whose keys are ifindices and whose values are references to arrays whose values are mac addresses associated with those ifindices via the devices CAM/forwarding table.  In code terms this looks like:  $$rh{ 'by_ifindex' }{ $ifindex } = \@macs

=item *
fetch_cam_table($type) 

Info:
  A wrapper function which attempts to cull the CAM/forwarding table from a device via SNMP.  Since different vendors have chosen to use different OID tables to provide this information, this function is designed as clearing house for all these OIDs, providing a single function to call which will return the cam table for a device either by calling the appropriate function based on the passed type or by trying to retrieve the cam table via any/all of the available functions.
    If no type is passed, the function will first try querying the host's cam table as if it were an Extreme Networks device.  If this does not produce results, the function will then try querying the device as if it were a Cisco Networks device (or any other device type supporting the 'dot1dTpFdbPort' OID table).  If this produces no results, an empty hash is returned.

Argument(s):

  $type (optional)

    The type of device that is being queried.
    Current valid values are: "cisco", "extreme"


Return value:

  returns a reference to a hash whose keys are 'by_mac' and 'by_ifindex'

  The value for 'by_mac' is a reference to a hash whose keys are mac addresses and whose values are the ifindices for those mac addresses as held in the device's CAM/forwarding table.  In code terms this looks like: $$rh{ 'by_mac' }{ $mac } = $ifindex

  The value for 'by_ifindex' is a reference to a hash whose keys are ifindices and whose values are references to arrays whose values are mac addresses associated with those ifindices via the devices CAM/forwarding table.  In code terms this looks like:  $$rh{ 'by_ifindex' }{ $ifindex } = \@macs


=item *
fetch_arp_table($ifindex) 

Info:
  Walks the ipNetToMediaPhysAddress table on a device.  ipNetToMediaPhysAddress contains the IP ARP table for a host.  For the really uninformed, the ARP table relates IP addresses to MAC Addresses.  This is especially useful information to collect from routers.
    ipNetToMediaPhysAddress is actually indexed like so: ifindex.A.B.C.D where ifindex is the ifindex of the port on which the arp entry was learned and A.B.C.D is an IP address.  The values for these OIDs are mac addresses in binary format.  fetch_arp_table() handles converting these binary values into Cisco IOS style mac addresses (ex. 00ff.eeaa.bbcc)

Argument(s):

  $ifindex (optional)

    The ifindex to collect ipNetToMediaPhysAddress entries for.
    If given, the return results are restricted to only arp entries learned on the passed ifindex.


Return value:

  returns a reference to a hash with two keys 'by_mac' and 'by_ip'.

  The value for 'by_mac' is a reference to a hash whose keys are mac addresses and whose values are IP addresses.  In code terms this looks like:  $$rh{ 'by_mac' }{ $mac } = $ip

  The value for 'by_ip' is a reference to a hash whose keys are IP addresses and whose values are mac addresses.  In code terms this looks like:  $$rh{ 'by_ip' }{ $ip } = $mac

=item *
mac_addr_to_oid($mac) 

Info:
  Helper function, converts mac address in Cisco IOS style (ex. 00ff.abcd.ef00) into an OID for appending to dot1dTpFdbPort.

Argument(s):

  $mac

    mac address to be converted.


Return value:

  returns an OID stub suitable for appending to dot1dTpFdbPort.

=item *
oid_to_mac_addr($macoid) 

Info:
  Converts from an OID stub taken from walking the dot1dTpFdbPort table into a Cisco IOS style mac address (ex. 00ff.abcd.ef00)

Argument(s):

  $macoid (optional)

    OID stub taken from walking the dot1dTpFdbPort table to be converted into a Cisco IOS style mac address (ex. 00ff.abcd.ef00)


Return value:

  returns a Cisco IOS style mac address (ex. 00ff.abcd.ef00)

=item *
unpack_mac($mac) 

Info:
  Takes a binary value for a mac address as found in the ipNetToMediaPhysAddress table and converts it into a Cisco IOS style mac address (ex. 00ff.abcd.ef00)

Argument(s):

  $mac

     binary value for a mac address as found in the ipNetToMediaPhysAddress table.

Return value:

  returns a Cisco IOS style mac address (ex. 00ff.abcd.ef00).

=item *
ucd_snmpwalk($oid, $vlan) 

Info:
  Calls the external program "/usr/bin/snmpwalk -Cc -Ofn " to perform an SNMP walk instead of using teh snmpwalk routine of the SNMP_Util module.  See the discussion of the $use_ucd_snmpwalk option to walk() for info on why one might want to do this.

Argument(s):

  $oid

    SNMP OID to be walked.

  $vlan (optional)

     Vlan number for use in an 'community@vlan' style SNMP query (see above discussion for get()).


Return value:

  returns results of an SNMP walk on the passed OID in the same format as walk().

=item *
consolidate_list($ra) 

Info:
  Helper function.  Pass it a reference to an array and it converts the array into a comma sperated list of numbers with consecutive values truncated with a dash.  For example, the array ( "1", "2", "3", "4", "7", "8", "9", "11" ) becomes te string "1-4,7,8-9,11".
  Useful when printing out long lists of vlans.

Argument(s):

  $ra

    reference to an array of numbers.


Return value:

  returns a scalar whose value is a comma sperated list of numbers with consecutive values truncated with a dash. (ie. "1-4,7,8-9,11")


=back

